# Copyright 2019 Mellanox Technologies, Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from networking_mlnx.internal.sys_ops import impl_os
from networking_mlnx import privsep

# for now we have only one implementation
__sys_ops = impl_os.OsCommand()


@privsep.admin_pctx.entrypoint
def sys_write(path, value):
    """Write to sysfs path (/sys/...)

    :param path: sysfs path
    :param value: value to write
    :raises InvalidSystemPathError: provided path is not under sysfs
    :raises SystemOperationError: an error occured when opening/writing
                                  to file.
    """
    __sys_ops.sys_write(path, value)
